// File: StockFilter.java

package stock;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

@WebFilter(urlPatterns = { "/StockService2", "/StockService3" })
public class StockFilter implements Filter {
  private FilterConfig filterConfig;
  @Override
  public void init(FilterConfig fg) throws ServletException {
    filterConfig = fg;
  }
  @Override
  public void destroy() {
    filterConfig = null;
  }
  @Override
  public void doFilter(ServletRequest request, ServletResponse response,
      FilterChain filterChain) throws IOException, ServletException {

    // If someone requests data with HP's old symbol ('hwp'), then 
    // modify the parameter to be 'hpq' instead
    if (request.getParameter("symbol").equals("hwp")) {
      MyHttpServletRequestWrapper wrapper = 
        new MyHttpServletRequestWrapper((HttpServletRequest) request);
      wrapper.setParameter("symbol", "hpq");
      request = wrapper;
    }
    filterChain.doFilter(request, response);
  }
}

class MyHttpServletRequestWrapper extends HttpServletRequestWrapper {
  Map<String, String[]> parameters;

  public MyHttpServletRequestWrapper(HttpServletRequest request) {
    super(request);
    parameters = new HashMap<String, String[]>();
    // load the data from the real parameter map into our map 
    parameters.putAll(super.getParameterMap());
  }
  // allow for new parameters or modifications to parameters 
  public void setParameter(String name, String value) {
    parameters.put(name, new String[] { value });
  }
  @Override
  public String getParameter(String name) {
    String s = parameters.get(name)[0];
    if (s == null) {
      s = super.getParameter(name);
    }
    return s;
  }
  @Override
  public Map<String, String[]> getParameterMap() {
    return parameters;
  }
  @Override
  public java.util.Enumeration<java.lang.String> getParameterNames() {
    return Collections.enumeration(parameters.keySet());
  }
  public String[] getParameterValues(String name) {
    return parameters.get(name);
  }
}
