// File: LibraryDAOImpl.java

package library;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Collection;
import javax.sql.DataSource;
import javax.naming.InitialContext;

public class LibraryDAOImpl implements LibraryDAO {

  public LibraryDAOImpl(){
  }

  public Collection<Book> getBooksByIsbn(String isbn) throws Exception {
    Connection conn = null;
    PreparedStatement pst = null;
    ResultSet rs = null;
    Collection<Book> books = null;

    String sqltxt = "SELECT b.isbn, b.title, b.call_number, " +
      "b.status, a.id, a.first_name, a.middle_name, a.last_name " +
      "FROM book b, author a, book_author ba " +
      "WHERE b.isbn = ba.isbn " +
      "and a.id = ba.id " +
      "and b.isbn=?";
    try {
      conn = getConnection();
      pst = conn.prepareStatement(sqltxt);
      pst.setString(1, isbn);
      rs = pst.executeQuery();

      books = addBooksToCollection(rs);
    }
    catch (Exception e) {
      System.err.println(e);
      throw e;
    }
    finally {
      rs.close();
      pst.close();
      conn.close();
    }
    return books;
  }

  public Collection<Book> getBooksByAuthor(String ln) throws Exception {
    Connection conn = null;
    PreparedStatement pst = null;
    ResultSet rs = null;
    Collection<Book> books = null;

    String sqltxt = "SELECT b.isbn, b.title, b.call_number, " +
      "b.status, a.id, a.first_name, a.middle_name, a.last_name " +
      "FROM book b, author a, book_author ba " +
      "WHERE b.isbn = ba.isbn " +
      "and a.id = ba.id " +
      "and a.last_name=?";
    try {
      conn = getConnection();
      pst = conn.prepareStatement(sqltxt);
      pst.setString(1, ln);
      rs = pst.executeQuery();

      books = addBooksToCollection(rs);
    }
    catch (Exception e) {
      System.err.println(e);
      throw e;
    }
    finally {
      rs.close();
      pst.close();
      conn.close();
    }
    return books;
  }

  public Collection<Book> getBooksByKeyword(String kw) throws Exception {
    Connection conn = null;
    PreparedStatement pst = null;
    ResultSet rs = null;
    Collection<Book> books = null;

    String sqltxt = "SELECT b.isbn, b.title, b.call_number, " +
      "b.status, a.id, a.first_name, a.middle_name, a.last_name " +
      "FROM book b, author a, book_author ba " +
      "WHERE b.isbn = ba.isbn " +
      "and a.id = ba.id " +
      "and b.title LIKE ?";

    try {
      conn = getConnection();
      pst = conn.prepareStatement(sqltxt);
      pst.setString(1, "%" + kw + "%");
      rs = pst.executeQuery();

      books = addBooksToCollection(rs);
    }
    catch (Exception e) {
      System.err.println(e);
      throw e;
    }
    finally {
      rs.close();
      pst.close();
      conn.close();
    }
    return books;
  }

  public Collection<Book> getBooksByTitle(String title) throws Exception {
    Connection conn = null;
    PreparedStatement pst = null;
    ResultSet rs = null;
    Collection<Book> books = null;

    String sqltxt = "SELECT b.isbn, b.title, b.call_number, " +
      "b.status, a.id, a.first_name, a.middle_name, a.last_name " +
      "FROM book b, author a, book_author ba " +
      "WHERE b.isbn = ba.isbn " +
      "and a.id = ba.id " +
      "and b.title = ?";

    try {
      conn = getConnection();
      pst = conn.prepareStatement(sqltxt);
      pst.setString(1, title);
      rs = pst.executeQuery();

      books = addBooksToCollection(rs);
    }
    catch (Exception e) {
      System.err.println(e);
      throw e;
    }
    finally {
      rs.close();
      pst.close();
      conn.close();
    }
    return books;
  }

  private Collection<Book> addBooksToCollection(ResultSet rs)
      throws Exception {
    Collection<Book> books = new ArrayList<Book>();
    Collection<Author> authors = null;
    Book book = null;
    String lastBookId = null;

    while (rs.next()) {
      String thisBookId = rs.getString("isbn");
      if (!thisBookId.equals(lastBookId)) {
        // first time we've encountered this book
        if (book != null) {
          // update and store old book
          book.setAuthors(authors);
          books.add(book);
        }

        book = new Book(thisBookId,
          rs.getString("call_number"),
          rs.getString("title"),
          rs.getString("status"));

        authors = new ArrayList<Author>();
      }

      Author a = new Author(rs.getInt("id"),
        rs.getString("first_name"),
        rs.getString("middle_name"),
        rs.getString("last_name"));

      authors.add(a);

      lastBookId = thisBookId;
    }
    if (book != null) {
      // update and store the final book
      book.setAuthors(authors);
      books.add(book);
    }

    return books;
  }

  private Connection getConnection() throws Exception {
    InitialContext ctxt = new InitialContext();
    DataSource ds =
      (DataSource) ctxt.lookup("java:comp/env/jdbc/books");
    Connection conn = ds.getConnection();
    return conn;
  }
}