// File: ProcessSearchServlet.java

package library;

import java.io.IOException;
import java.util.Collection;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@WebServlet(urlPatterns = {"/ProcessSearch"})
public class ProcessSearchServlet extends HttpServlet {
  @Override
  public void doGet(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException {

    String searchType = req.getParameter("searchType");
    String searchString = req.getParameter("searchString");

    LibraryDAO dao = new LibraryDAOImpl();
    Collection<Book> books = null;

    try {
      if (searchType.equals("isbn")) {
        books = dao.getBooksByIsbn(searchString);
      }
      else if (searchType.equals("author")) {
        books = dao.getBooksByAuthor(searchString);
      }
      else if (searchType.equals("subject")) {
        books = dao.getBooksByKeyword(searchString);
      }
      else if (searchType.equals("title")) {
        books = dao.getBooksByTitle(searchString);
      }

      if (books == null || books.size() == 0) {
        forwardToJSP(req, res, "/DisplayNoResults.jsp");
      }
      else {
        req.setAttribute("book", books.iterator().next());
        forwardToJSP(req, res, "/DisplayResults.jsp");
      }
    }
    catch (Exception e) {
      req.setAttribute("exception", e);
      forwardToJSP(req, res, "/DisplayErrors.jsp");
    }

  }

  @Override
  public void doPost(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException {
    doGet(req, res);
  }

  private void forwardToJSP(HttpServletRequest request,
    HttpServletResponse response, String jsp)
      throws ServletException, IOException {

    ServletContext context = getServletContext();
    RequestDispatcher dispatcher = context.getRequestDispatcher(jsp);
    dispatcher.forward(request, response);
  }
}

