// File: ArticleFactory.java

package blognews;

import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.File;
import java.util.ArrayList;

public class ArticleFactory {
  private File archive;

  public ArticleFactory(File archive) {
    this.archive = archive;
  }

  public Article getArticle(String title)
      throws ArticleNotFoundException {
    File articleFile = new File(archive, title);
    return getArticle(articleFile);
  }

  public void writeArticle(Article article) throws IOException {
    ObjectOutputStream out = null;
    File articleFile = new File(archive, article.getTitle());

    try {
      out = new ObjectOutputStream(new FileOutputStream(articleFile));
      out.writeObject(article);
    }
    finally {
      if (out != null) {
        try {
          out.close();
        }
        catch (IOException ignored) {}
      }
    }
  }

  public ArrayList<Article> getArticleList() {
    File[] files = archive.listFiles();
    ArrayList<Article> list = new ArrayList<Article>(files.length);

    for (int i = 0; i < files.length; i++) {
      try {
        list.add(getArticle(files[i]));
      }
      catch (ArticleNotFoundException ignored) {}
    }
    return list;
  }

  private Article getArticle(File articleFile)
    throws ArticleNotFoundException {
    Article result;

    if (!articleFile.canRead()) {
      throw new ArticleNotFoundException(articleFile.getName());
    }

    FileInputStream fin = null;
    ObjectInputStream in = null;
    try {
      fin = new FileInputStream(articleFile);
      in = new ObjectInputStream(fin);
      result = (Article)in.readObject();
    }
    catch (Exception ex) {
      throw new ArticleNotFoundException(
          "Error reading " + articleFile.getName() + ": " + ex, ex);
    }
    finally {
      if (in != null) {
        try {
          in.close();
        }
        catch (IOException ignored) {}
      }
      else if (fin != null) {
        try {
          fin.close();
        }
        catch (IOException ignored) {}
      }
    }
    return result;
  }
}
