// File: CheckoutServlet.java

package com.holidaytruffles;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.*;

@WebServlet(urlPatterns = {"/CheckoutServlet"})
public class CheckoutServlet extends HttpServlet {
  @Override
  public void doGet(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException {
    ServletContext ctx = getServletContext();
    Map<String, Double> prices = (Map<String, Double>) ctx.getAttribute("prices");
    HttpSession session = req.getSession();
    List<Item> cart = (List<Item>) session.getAttribute("cart");
    CartBean cb = new CartBean();

    if (cart != null && cart.size() != 0) {
      for (Item item : cart) {
        int q = item.getQuantity();
        double price = prices.get(item.getCode());

        cb.addLineItem(item.getName(), price, q, q * price);
      }
      cart.clear();
      session.setAttribute("cart", cart);
      req.setAttribute("cartBean", cb);
    }

    RequestDispatcher dispatcher = ctx.getRequestDispatcher("/ProcessCheckout.jsp");

    dispatcher.forward(req, res);
  }

  @Override
  public void init() {
    // Normally you would load the prices from a database.
    ServletContext ctx = getServletContext();
    Map<String, Double> prices = (Map<String, Double>) ctx.getAttribute("prices");
    if (prices == null) {
      prices = new HashMap<String, Double>();
      prices.put("Am", 2.0);
      prices.put("Ch", 2.0);
      prices.put("GM", 2.0);
      prices.put("IC", 2.0);
      prices.put("Pe", 2.0);
      prices.put("Ra", 2.0);
      prices.put("Ru", 2.0);
      ctx.setAttribute("prices", prices);
    }
  }
}
