// File: PriceServlet.java

package com.holidaytruffles;

import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.*;

@WebServlet(urlPatterns = {"/admin/SetPrice"})
public class PriceServlet extends HttpServlet {
  @Override
  public void doGet(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException
  {
    Util util = new Util(req, res);
    List<PriceItem> priceBean = new ArrayList<PriceItem>();

    for (String code : util.getPrices().keySet())
      priceBean.add(new PriceItem(Item.getName(code), code, util.getFormattedPrice(code)));
    req.setAttribute("priceBean", priceBean);
    util.forward("/admin/SetPrice.jsp");
  }

  @Override
  public void doPost(HttpServletRequest req, HttpServletResponse res)
    throws ServletException, IOException
  {
    Util util = new Util(req, res);

    try {
      util.parseParm("Am");
      util.parseParm("Ch");
      util.parseParm("GM");
      util.parseParm("IC");
      util.parseParm("Pe");
      util.parseParm("Ra");
      util.parseParm("Ru");
    } catch (ParseException e) {
      throw new ServletException(e);
    }
    util.forward("/admin/ProcessSetPrice.jsp");
  }

  @Override
  public void init() {
    // Normally you would load the prices from a database.
    ServletContext ctx = getServletContext();
    Map<String, Double> prices = (Map<String, Double>) ctx.getAttribute("prices");
    if (prices == null) {
      prices = new HashMap<String, Double>();
      prices.put("Am", 2.0);
      prices.put("Ch", 2.0);
      prices.put("GM", 2.0);
      prices.put("IC", 2.0);
      prices.put("Pe", 2.0);
      prices.put("Ra", 2.0);
      prices.put("Ru", 2.0);
      ctx.setAttribute("prices", prices);
    }
  }

  private class Util {
    private ServletContext ctx;
    private NumberFormat format;
    private Map<String, Double> prices;
    private HttpServletRequest req;
    private HttpServletResponse res;

    Util(HttpServletRequest req, HttpServletResponse res) {
      this.req = req;
      this.res = res;
      ctx = getServletContext();
      prices = (Map<String, Double>) ctx.getAttribute("prices");
      String currency = ctx.getInitParameter("currency");
      Locale locale = Locale.US;

      if (currency.equals("GBP"))
        locale = Locale.UK;
      else if (currency.equals("JPY"))
        locale = Locale.JAPAN;
      else if (currency.equals("EUR"))
        locale = Locale.FRANCE;
      format = NumberFormat.getCurrencyInstance(locale);
    }

    String getFormattedPrice(String code) {
      return format.format(prices.get(code));
    }

    void forward(String url) throws ServletException, IOException {
      ctx.getRequestDispatcher(url).forward(req, res);
    }

    Map<String, Double> getPrices() {
      return prices;
    }

    void parseParm(String parm) throws ParseException
    {
      prices.put(parm, format.parse((String) req.getParameter(parm)).doubleValue());
    }
  }
}
